     /*   
     Arduino code for simple dashboard with a doudle 74HC595 shitf register and 8x 7 segmet led
     created 27 Mar 2015 by Soraz
     */

    // USO command to play : M~a03~or S~a04~ or G~a05~...
    // USO command to simulatorstop : Z00
    // M : rpm              (16bit - binary output)
    // S : speed            (16bit - binary output)
    // G : gear             (16bit - binary output)
    // O : oil temp         (16bit - binary output)
    // W : water temp       (16bit - binary output)
    // F : fuel             (16bit - binary output)
    // L : lap              (16bit - binary output)
    // P : position         (16bit - binary output)
    // Z : clear            (to clear display and max rpm value)
    // 

    // for dashboard
    // Pin D10 : RCK-STB0 - strobe - latchPin
    // Pin D11 : SCK-CLK - clock -  clockPin 
    // Pin D12 : DIO-DIO - data - dataPin 
    #include <HC595.h>
    #include <HC595Font.h>
    HC595 module;
    
    // for dashboard   
    int i;
    unsigned long tt;           // holds milliseconds 
    unsigned int lastval;       // holds the last val to repeat
    char* lastchar;             // holds the last char to repeat
    unsigned int offset = -32767; // holds offset for dashboard
    unsigned int carspeed;      // holds the speed data (0-65535 size)
    unsigned int gear;          // holds gear value data (0-65535 size)
    unsigned int rpm;           // holds the rpm data (0-65535 size)
    unsigned int oil;           // holds the oil temp data (0-65535 size)
    unsigned int water;         // holds the water temp data (0-65535 size)
    unsigned int fuel;          // holds the fuel data (0-65535 size)
    unsigned int lap;           // holds the lap data (0-65535 size)
    unsigned int posit;         // holds the position data (0-65535 size)
            
    unsigned int rpmmax = 5000;    // retrieves from rpm live with minimum setting on initialization and simulatorstop
    unsigned int setPos = 0;       // holds the display position
    
    byte speeddata = 0;         // marker that new data are available
    byte geardata = 0;          // marker that new data are available   
    byte rpmdata = 0;           // marker that new data are available
    byte oildata = 0;           // marker that new data are available
    byte waterdata = 0;         // marker that new data are available
    byte fueldata = 0;          // marker that new data are available
    byte lapdata = 0;           // marker that new data are available
    byte positiondata = 0;      // marker that new data are available

    // char* neutral = "n";     // sets the character for neutral
    char* neutral = "0";        // sets the character for neutral
    char* reverse = "r";        // sets the character for reverse
    
    char* namesz = "88888888"; // sets a custom logo start up banner


    ////////////////////////////////////////////////////////////////////////////////
    // INITIALIZATION
    ////////////////////////////////////////////////////////////////////////////////
    void setup()
    {
     
      // serial initialization
      Serial.begin(115200);

      lastchar="";
      lastval=0;
      
      // on all led
      module.clear();
      // on all dots
      module.setDots(0xFF);
      tt=millis();
      while(tt+999>millis()) {
        module.setChars(namesz);
      }
      module.setDots(0);
      module.clear();
      
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////// Main Loop ////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////
    void loop()
    ///////////////////////////////////////////////////////////////////////////////    
    {

      // read serial data
      readSerialData();   

      // update dashboard
      dashboard();
      
    }

    ////////////////////////////////////////////////////////////////////////////////
    // Procedure: read serial data
    ////////////////////////////////////////////////////////////////////////////////
    void readSerialData()
    ////////////////////////////////////////////////////////////////////////////////
    {
      byte Data[3]={'0','0','0'};
      // keep this function short, because the loop has to be short to keep the control over the motors

      // parse the buffer
      if (Serial.available()>2){
        Data[0]=Serial.read();                   // read command
        if (Data[0]=='M'){                       // if rpm data 
          Data[1]=Serial.read();                 // store high byte
          Data[2]=Serial.read();                 // store low byte
          rpm = ((Data[1]<<8) + Data[2]);        // concatonate bytes (shift 8 bits)
          rpm = rpm + offset;                    // offset
          rpmdata = 1;                           // we got new data!        
          if (Serial.available()>2){
            Data[0]=Serial.read();               // re-read command
          }
        }
        if (Data[0] == 'S' ){                    // if speed data
          Data[1]=Serial.read();                 // store high byte
          Data[2]=Serial.read();                 // store low byte
          carspeed = ((Data[1]<<8) + Data[2]);   // concatonate bytes (shift 8 bits)
          carspeed = carspeed + offset;          // offset
          speeddata = 1;                         // we got new data!
          if (Serial.available()>2){
            Data[0]=Serial.read();               // re-read command
          }
        }
        if (Data[0] == 'G' ){                    // if gear data 
          Data[1]=Serial.read();                 // store high byte
          Data[2]=Serial.read();                 // store low byte
          gear=((Data[1]<<8) + Data[2]);         // concatonate bytes (shift 8 bits)
          gear = gear + offset;                  // offset
          geardata = 1;                          // we got new data!
          if (Serial.available()>2){
            Data[0]=Serial.read();               // re-read command
          }
        }              
        if (Data[0] == 'O' ){                    // if oil data
          Data[1]=Serial.read();                 // store high byte
          Data[2]=Serial.read();                 // store low byte
          oil=((Data[1]<<8) + Data[2]);          // concatonate bytes (shift 8 bits)
          oil = oil + offset;                    // offset
          oildata = 1;                           // we got new data!
          if (Serial.available()>2){
            Data[0]=Serial.read();               // re-read command
          }
        }             
        if (Data[0] == 'W' ){                    // if water data
          Data[1]=Serial.read();                 // store high byte
          Data[2]=Serial.read();                 // store low byte
          water=((Data[1]<<8) + Data[2]);        // concatonate bytes (shift 8 bits)
          water = water + offset;                // offset
          waterdata = 1;                         // we got new data!
          if (Serial.available()>2){
            Data[0]=Serial.read();               // re-read command
          }
        }             
        if (Data[0] == 'F' ){                    // if fuel data
          Data[1]=Serial.read();                 // store high byte
          Data[2]=Serial.read();                 // store low byte
          fuel=((Data[1]<<8) + Data[2]);         // concatonate bytes (shift 8 bits)
          fuel = fuel + offset;                  // offset
          fueldata = 1;                          // we got new data!
          if (Serial.available()>2){
            Data[0]=Serial.read();               // re-read command
          }
        }             
        if (Data[0] == 'L' ){                    // if lap data
          Data[1]=Serial.read();                 // store high byte
          Data[2]=Serial.read();                 // store low byte
          lap=((Data[1]<<8) + Data[2]);          // concatonate bytes (shift 8 bits)
          lap = lap + offset;                    // offset
          lapdata = 1;                           // we got new data!
          if (Serial.available()>2){
            Data[0]=Serial.read();               // re-read command
          }
        }             
        if (Data[0] == 'P' ){                    // if position data
          Data[1]=Serial.read();                 // store high byte
          Data[2]=Serial.read();                 // store low byte
          posit=((Data[1]<<8) + Data[2]);        // concatonate bytes (shift 8 bits)
          posit = posit + offset;                // offset
          positiondata = 1;                      // we got new data!
          if (Serial.available()>2){
            Data[0]=Serial.read();               // re-read command
          }
        }             
        
        if (Data[0] == 'Z' ){                          // if simulatorstop 
          Data[1]=Serial.read();                       // store high byte
          Data[2]=Serial.read();                       // store low byte        
          rpmmax=5000;                                 // reset rpmmax
          module.clear();                              // clear display    
          lastval=0;
          lastchar="";
        }              
      }
      if (Serial.available()>30) Serial.flush();
    }

    ////////////////////////////////////////////////////////
    void dashboard()
    ////////////////////////////////////////////////////////
    {    
      if (speeddata == 1) {
        // filter of negative value
        // reported from 65535 to down in some games
        if (carspeed > 50000){                            // reverse velocity in some games are negative value
          carspeed = 65536 - carspeed;                    // convert positive with 16bit value
        }
        lastval=carspeed;              // displays numerical the speed
        // if (carspeed == 0){
        //   module.setChars("0",7);
        // }            
        speeddata=0;           
      } 
      if (geardata == 1) {
          setPos = 7;
          if (gear >= 1 and gear <10 ){         
            lastval=gear;                        // displays numerical value of the current gear
            lastchar="";   
          }
          if (gear == 0){
            lastchar=neutral;
            module.setChars(neutral,7);                      // displays the character for neutral
          }
          if (gear == (65535)){                                   // -1 that reprecents reverse rollover so...
            lastchar=reverse;
            module.setChars(reverse,7);                      // displays the character for reverse
          }
        geardata=0;
      } 
      if (rpmdata == 1) {
        lastval=rpm;
        // auto set max rpm
        // with filter of negative value
        // reported from 65535 to down in some games
        if (rpm>rpmmax && rpm<50000) {
          rpmmax=rpm;
        }
        rpmdata=0;
      }
      if (oildata == 1) {
        lastval=oil;
        oildata=0;
      } 
      if (waterdata == 1) {
        lastval=water;
        waterdata=0;           
      } 
      if (fueldata == 1) {
        lastval=fuel;
        fueldata=0;           
      } 
      if (lapdata == 1) {
        lastval=lap;
        lapdata=0;
      } 
      if (positiondata == 1) {
        lastval=posit;
        positiondata=0;
      } 
      // repeat continuosly last value
      if (lastchar=="") {
          module.setNumber(lastval);
        }
      else {
          module.setChars(lastchar,7);
      }
      //
     }
